from django.contrib import admin
from django.urls import include, path

from . import views

urlpatterns = [
    path("", views.BookCreateView.as_view(), name="book-create"),
    path("select2/", include("django_select2.urls")),
    path("admin/", admin.site.urls),
]



import json

from django.utils.encoding import smart_str

from django_select2.cache import cache
from django_select2.forms import ModelSelect2Widget
from tests.testapp.forms import AlbumModelSelect2WidgetForm, ArtistCustomTitleWidget
from tests.testapp.models import Genre

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse


class TestAutoResponseView:
    def test_get(self, client, artists):
        artist = artists[0]
        form = AlbumModelSelect2WidgetForm()
        assert form.as_p()
        field_id = form.fields["artist"].widget.field_id
        url = reverse("django_select2:auto-json")
        response = client.get(url, {"field_id": field_id, "term": artist.title})
        assert response.status_code == 200
        data = json.loads(response.content.decode("utf-8"))
        assert data["results"]
        assert {"id": artist.pk, "text": smart_str(artist)} in data["results"]

    def test_no_field_id(self, client, artists):
        artist = artists[0]
        url = reverse("django_select2:auto-json")
        response = client.get(url, {"term": artist.title})
        assert response.status_code == 404

    def test_wrong_field_id(self, client, artists):
        artist = artists[0]
        url = reverse("django_select2:auto-json")
        response = client.get(url, {"field_id": 123, "term": artist.title})
        assert response.status_code == 404

    def test_field_id_not_found(self, client, artists):
        artist = artists[0]
        field_id = "not-exists"
        url = reverse("django_select2:auto-json")
        response = client.get(url, {"field_id": field_id, "term": artist.title})
        assert response.status_code == 404

    def test_pagination(self, genres, client):
        url = reverse("django_select2:auto-json")
        widget = ModelSelect2Widget(
            max_results=10, model=Genre, search_fields=["title__icontains"]
        )
        widget.render("name", None)
        field_id = widget.field_id

        response = client.get(url, {"field_id": field_id, "term": ""})
        assert response.status_code == 200
        data = json.loads(response.content.decode("utf-8"))
        assert data["more"] is True

        response = client.get(url, {"field_id": field_id, "term": "", "page": 1000})
        assert response.status_code == 404

        response = client.get(url, {"field_id": field_id, "term": "", "page": "last"})
        assert response.status_code == 200
        data = json.loads(response.content.decode("utf-8"))
        assert data["more"] is False

    def test_label_from_instance(self, artists, client):
        url = reverse("django_select2:auto-json")

        form = AlbumModelSelect2WidgetForm()
        form.fields["artist"].widget = ArtistCustomTitleWidget()
        assert form.as_p()
        field_id = form.fields["artist"].widget.field_id

        artist = artists[0]
        response = client.get(url, {"field_id": field_id, "term": artist.title})
        assert response.status_code == 200

        data = json.loads(response.content.decode("utf-8"))
        assert data["results"]
        assert {"id": artist.pk, "text": smart_str(artist.title.upper())} in data[
            "results"
        ]

    def test_url_check(self, client, artists):
        artist = artists[0]
        form = AlbumModelSelect2WidgetForm()
        assert form.as_p()
        field_id = form.fields["artist"].widget.field_id
        cache_key = form.fields["artist"].widget._get_cache_key()
        widget_dict = cache.get(cache_key)
        widget_dict["url"] = "yet/another/url"
        cache.set(cache_key, widget_dict)
        url = reverse("django_select2:auto-json")
        response = client.get(url, {"field_id": field_id, "term": artist.title})
        assert response.status_code == 404



from django.db import models


class Genre(models.Model):
    title = models.CharField(max_length=50)

    class Meta:
        ordering = ("title",)

    def __str__(self):
        return self.title


class Artist(models.Model):
    title = models.CharField(max_length=50, unique=True)
    genres = models.ManyToManyField(Genre)

    class Meta:
        ordering = ("title",)

    def __str__(self):
        return self.title


class Album(models.Model):
    title = models.CharField(max_length=255)
    artist = models.ForeignKey(Artist, on_delete=models.CASCADE)
    featured_artists = models.ManyToManyField(
        Artist, blank=True, related_name="featured_album_set"
    )
    primary_genre = models.ForeignKey(
        Genre,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name="primary_album_set",
    )
    genres = models.ManyToManyField(Genre)

    class Meta:
        ordering = ("title",)

    def __str__(self):
        return self.title


class Country(models.Model):
    name = models.CharField(max_length=255)

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name


class City(models.Model):
    name = models.CharField(max_length=255)
    country = models.ForeignKey(
        "Country", related_name="cities", on_delete=models.CASCADE
    )

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name


class Groupie(models.Model):
    obsession = models.ForeignKey(Artist, to_field="title", on_delete=models.CASCADE)



import os
import pathlib
import sys

from pkg_resources import get_distribution

BASE_DIR = pathlib.Path(__file__).resolve(strict=True).parent.parent

# This is needed since django_select2 requires django model modules
# and those modules assume that django settings is configured and
# have proper DB settings.
# Using this we give a proper environment with working django settings.
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.testapp.settings")

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, str(BASE_DIR / "tests" / "testapp"))
sys.path.insert(0, str(BASE_DIR))


project = "Django-Select2"
author = "Johannes Hoppe"
copyright = "2017-2020, Johannes Hoppe"
release = get_distribution("django_select2").version
version = ".".join(release.split(".")[:2])


master_doc = "index"  # default in Sphinx v2


extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx.ext.inheritance_diagram",
    "sphinx.ext.intersphinx",
    "sphinx.ext.viewcode",
    "sphinx.ext.doctest",
]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "django": (
        "https://docs.djangoproject.com/en/stable/",
        "https://docs.djangoproject.com/en/stable/_objects/",
    ),
}

autodoc_default_flags = ["members", "show-inheritance"]
autodoc_member_order = "bysource"

inheritance_graph_attrs = dict(rankdir="TB")

inheritance_node_attrs = dict(
    shape="rect", fontsize=14, fillcolor="gray90", color="gray30", style="filled"
)

inheritance_edge_attrs = dict(penwidth=0.75)



import os.path

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
DEBUG = True

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "django_select2",
    "tests.testapp",
)

STATIC_URL = "/static/"

MEDIA_ROOT = os.path.join(BASE_DIR, "media")

SITE_ID = 1
ROOT_URLCONF = "tests.testapp.urls"

LANGUAGES = [
    ("de", "German"),
    ("en", "English"),
]
LANGUAGE_CODE = "en"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "DIRS": ["templates"],
    },
]

SECRET_KEY = "123456"

USE_L10N = True
USE_I18N = True



import random
import string

import pytest
from selenium import webdriver
from selenium.common.exceptions import WebDriverException


def random_string(n):
    return "".join(
        random.choice(string.ascii_uppercase + string.digits) for _ in range(n)
    )


def random_name(n):
    words = (
        "".join(random.choice(string.ascii_lowercase + " ") for _ in range(n))
        .strip()
        .split(" ")
    )
    return "-".join([x.capitalize() for x in words])


@pytest.fixture(scope="session")
def driver():
    chrome_options = webdriver.ChromeOptions()
    chrome_options.headless = True
    try:
        b = webdriver.Chrome(options=chrome_options)
    except WebDriverException as e:
        pytest.skip(str(e))
    else:
        yield b
        b.quit()


@pytest.fixture
def genres(db):
    from .testapp.models import Genre

    return Genre.objects.bulk_create(
        [Genre(pk=pk, title=random_string(50)) for pk in range(100)]
    )


@pytest.fixture
def artists(db):
    from .testapp.models import Artist

    return Artist.objects.bulk_create(
        [Artist(pk=pk, title=random_string(50)) for pk in range(100)]
    )


@pytest.fixture
def countries(db):
    from .testapp.models import Country

    return Country.objects.bulk_create(
        [Country(pk=pk, name=random_name(random.randint(10, 20))) for pk in range(10)]
    )


@pytest.fixture
def cities(db, countries):
    from .testapp.models import City

    return City.objects.bulk_create(
        [
            City(
                pk=pk,
                name=random_name(random.randint(5, 15)),
                country=random.choice(countries),
            )
            for pk in range(100)
        ]
    )



from django.urls import include, path

from .forms import (
    AddressChainedSelect2WidgetForm,
    AlbumModelSelect2WidgetForm,
    HeavySelect2MultipleWidgetForm,
    HeavySelect2WidgetForm,
    ModelSelect2TagWidgetForm,
    Select2WidgetForm,
)
from .views import TemplateFormView, heavy_data_1, heavy_data_2

urlpatterns = [
    path(
        "select2_widget",
        TemplateFormView.as_view(form_class=Select2WidgetForm),
        name="select2_widget",
    ),
    path(
        "heavy_select2_widget",
        TemplateFormView.as_view(form_class=HeavySelect2WidgetForm),
        name="heavy_select2_widget",
    ),
    path(
        "heavy_select2_multiple_widget",
        TemplateFormView.as_view(
            form_class=HeavySelect2MultipleWidgetForm, success_url="/"
        ),
        name="heavy_select2_multiple_widget",
    ),
    path(
        "model_select2_widget",
        TemplateFormView.as_view(form_class=AlbumModelSelect2WidgetForm),
        name="model_select2_widget",
    ),
    path(
        "model_select2_tag_widget",
        TemplateFormView.as_view(form_class=ModelSelect2TagWidgetForm),
        name="model_select2_tag_widget",
    ),
    path(
        "model_chained_select2_widget",
        TemplateFormView.as_view(form_class=AddressChainedSelect2WidgetForm),
        name="model_chained_select2_widget",
    ),
    path("heavy_data_1", heavy_data_1, name="heavy_data_1"),
    path("heavy_data_2", heavy_data_2, name="heavy_data_2"),
    path("select2/", include("django_select2.urls")),
]



import json

from django.http import HttpResponse
from django.views.generic import FormView


class TemplateFormView(FormView):
    template_name = "form.html"


def heavy_data_1(request):
    term = request.GET.get("term", "")
    numbers = ["Zero", "One", "Two", "Three", "Four", "Five"]
    numbers = filter(lambda num: term.lower() in num.lower(), numbers)
    results = [{"id": index, "text": value} for (index, value) in enumerate(numbers)]
    return HttpResponse(
        json.dumps({"err": "nil", "results": results}), content_type="application/json"
    )


def heavy_data_2(request):
    term = request.GET.get("term", "")
    numbers = ["Six", "Seven", "Eight", "Nine", "Ten", "Fortytwo"]
    numbers = filter(lambda num: term.lower() in num.lower(), numbers)
    results = [{"id": index, "text": value} for (index, value) in enumerate(numbers)]
    return HttpResponse(
        json.dumps({"err": "nil", "results": results}), content_type="application/json"
    )



from django import forms
from django.utils.encoding import force_str

from django_select2.forms import (
    HeavySelect2MultipleWidget,
    HeavySelect2Widget,
    ModelSelect2MultipleWidget,
    ModelSelect2TagWidget,
    ModelSelect2Widget,
    Select2MultipleWidget,
    Select2Widget,
)
from tests.testapp import models
from tests.testapp.models import Album, City, Country


class TitleSearchFieldMixin:
    search_fields = ["title__icontains", "pk__startswith"]


class TitleModelSelect2Widget(TitleSearchFieldMixin, ModelSelect2Widget):
    pass


class TitleModelSelect2MultipleWidget(
    TitleSearchFieldMixin, ModelSelect2MultipleWidget
):
    pass


class GenreSelect2TagWidget(TitleSearchFieldMixin, ModelSelect2TagWidget):
    model = models.Genre

    def create_value(self, value):
        self.get_queryset().create(title=value)


class ArtistCustomTitleWidget(ModelSelect2Widget):
    model = models.Artist
    search_fields = ["title__icontains"]

    def label_from_instance(self, obj):
        return force_str(obj.title).upper()


class GenreCustomTitleWidget(ModelSelect2Widget):
    model = models.Genre
    search_fields = ["title__icontains"]

    def label_from_instance(self, obj):
        return force_str(obj.title).upper()


class AlbumSelect2WidgetForm(forms.ModelForm):
    class Meta:
        model = models.Album
        fields = (
            "artist",
            "primary_genre",
        )
        widgets = {
            "artist": Select2Widget,
            "primary_genre": Select2Widget,
        }


class AlbumSelect2MultipleWidgetForm(forms.ModelForm):
    class Meta:
        model = models.Album
        fields = (
            "genres",
            "featured_artists",
        )
        widgets = {
            "genres": Select2MultipleWidget,
            "featured_artists": Select2MultipleWidget,
        }


class AlbumModelSelect2WidgetForm(forms.ModelForm):
    class Meta:
        model = models.Album
        fields = (
            "artist",
            "primary_genre",
        )
        widgets = {
            "artist": ArtistCustomTitleWidget(),
            "primary_genre": GenreCustomTitleWidget(),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["primary_genre"].initial = 2


class AlbumModelSelect2MultipleWidgetRequiredForm(forms.ModelForm):
    class Meta:
        model = Album
        fields = (
            "genres",
            "featured_artists",
        )
        widgets = {
            "genres": TitleModelSelect2MultipleWidget,
            "featured_artists": TitleModelSelect2MultipleWidget,
        }


class ArtistModelSelect2MultipleWidgetForm(forms.Form):
    title = forms.CharField(max_length=50)
    genres = forms.ModelMultipleChoiceField(
        widget=ModelSelect2MultipleWidget(
            queryset=models.Genre.objects.all(),
            search_fields=["title__icontains"],
        ),
        queryset=models.Genre.objects.all(),
        required=True,
    )

    featured_artists = forms.ModelMultipleChoiceField(
        widget=ModelSelect2MultipleWidget(
            queryset=models.Artist.objects.all(),
            search_fields=["title__icontains"],
        ),
        queryset=models.Artist.objects.all(),
        required=False,
    )


NUMBER_CHOICES = [
    (1, "One"),
    (2, "Two"),
    (3, "Three"),
    (4, "Four"),
]


class Select2WidgetForm(forms.Form):
    number = forms.ChoiceField(
        widget=Select2Widget, choices=NUMBER_CHOICES, required=False
    )


class HeavySelect2WidgetForm(forms.Form):
    artist = forms.ChoiceField(
        widget=HeavySelect2Widget(data_view="heavy_data_1"), choices=NUMBER_CHOICES
    )
    primary_genre = forms.ChoiceField(
        widget=HeavySelect2Widget(data_view="heavy_data_2"),
        required=False,
        choices=NUMBER_CHOICES,
    )


class HeavySelect2MultipleWidgetForm(forms.Form):
    title = forms.CharField(max_length=50)
    genres = forms.MultipleChoiceField(
        widget=HeavySelect2MultipleWidget(
            data_view="heavy_data_1",
            choices=NUMBER_CHOICES,
            attrs={"data-minimum-input-length": 0},
        ),
        choices=NUMBER_CHOICES,
    )
    featured_artists = forms.MultipleChoiceField(
        widget=HeavySelect2MultipleWidget(
            data_view="heavy_data_2",
            choices=NUMBER_CHOICES,
            attrs={"data-minimum-input-length": 0},
        ),
        choices=NUMBER_CHOICES,
        required=False,
    )

    def clean_title(self):
        if len(self.cleaned_data["title"]) < 3:
            raise forms.ValidationError("Title must have more than 3 characters.")
        return self.cleaned_data["title"]


class ModelSelect2TagWidgetForm(forms.ModelForm):
    class Meta:
        model = Album
        fields = ["genres"]
        widgets = {"genres": GenreSelect2TagWidget}


class AddressChainedSelect2WidgetForm(forms.Form):
    country = forms.ModelChoiceField(
        queryset=Country.objects.all(),
        label="Country",
        widget=ModelSelect2Widget(
            search_fields=["name__icontains"],
            max_results=500,
            dependent_fields={"city": "cities"},
            attrs={"data-minimum-input-length": 0},
        ),
    )

    city = forms.ModelChoiceField(
        queryset=City.objects.all(),
        label="City",
        widget=ModelSelect2Widget(
            search_fields=["name__icontains"],
            dependent_fields={"country": "country"},
            max_results=500,
            attrs={"data-minimum-input-length": 0},
        ),
    )

    city2 = forms.ModelChoiceField(
        queryset=City.objects.all(),
        label="City not Interdependent",
        widget=ModelSelect2Widget(
            search_fields=["name__icontains"],
            dependent_fields={"country": "country"},
            max_results=500,
            attrs={"data-minimum-input-length": 0},
        ),
    )


class GroupieForm(forms.ModelForm):
    class Meta:
        model = models.Groupie
        fields = "__all__"
        widgets = {"obsession": ArtistCustomTitleWidget}



"""Django application configuration."""
from django.apps import AppConfig


class Select2AppConfig(AppConfig):
    """Django application configuration."""

    name = "django_select2"
    verbose_name = "Select2"



import json
import os
from collections.abc import Iterable

import pytest
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.db.models import QuerySet
from django.urls import reverse
from django.utils import translation
from django.utils.encoding import force_str
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait

from django_select2.cache import cache
from django_select2.conf import settings
from django_select2.forms import (
    HeavySelect2MultipleWidget,
    HeavySelect2Widget,
    ModelSelect2TagWidget,
    ModelSelect2Widget,
    Select2Widget,
    Select2AdminMixin,
)
from tests.testapp import forms
from tests.testapp.forms import (
    NUMBER_CHOICES,
    HeavySelect2MultipleWidgetForm,
    TitleModelSelect2Widget,
)
from tests.testapp.models import Artist, City, Country, Genre, Groupie


class TestSelect2Mixin:
    url = reverse("select2_widget")
    form = forms.AlbumSelect2WidgetForm()
    multiple_form = forms.AlbumSelect2MultipleWidgetForm()
    widget_cls = Select2Widget

    def test_initial_data(self, genres):
        genre = genres[0]
        form = self.form.__class__(initial={"primary_genre": genre.pk})
        assert str(genre) in form.as_p()

    def test_initial_form_class(self):
        widget = self.widget_cls(attrs={"class": "my-class"})
        assert "my-class" in widget.render("name", None)
        assert "django-select2" in widget.render("name", None)

    @pytest.mark.parametrize("code,name", SELECT2_TRANSLATIONS.items())
    def test_lang_attr(self, code, name):
        translation.activate(code)
        widget = self.widget_cls()
        assert f'lang="{name}"' in widget.render("name", None)

    def test_allow_clear(self, db):
        required_field = self.form.fields["artist"]
        assert required_field.required is True
        assert 'data-allow-clear="true"' not in required_field.widget.render(
            "artist", None
        )
        assert 'data-allow-clear="false"' in required_field.widget.render(
            "artist", None
        )
        assert '<option value=""></option>' not in required_field.widget.render(
            "artist", None
        )

        not_required_field = self.form.fields["primary_genre"]
        assert not_required_field.required is False
        assert 'data-allow-clear="true"' in not_required_field.widget.render(
            "primary_genre", None
        )
        assert 'data-allow-clear="false"' not in not_required_field.widget.render(
            "primary_genre", None
        )
        assert "data-placeholder" in not_required_field.widget.render(
            "primary_genre", None
        )
        assert '<option value=""></option>' in not_required_field.widget.render(
            "primary_genre", None
        )

    def test_no_js_error(self, db, live_server, driver):
        driver.get(live_server + self.url)
        with pytest.raises(NoSuchElementException):
            error = driver.find_element_by_xpath("//body[@JSError]")
            pytest.fail(error.get_attribute("JSError"))

    def test_selecting(self, db, live_server, driver):
        driver.get(live_server + self.url)
        with pytest.raises(NoSuchElementException):
            driver.find_element_by_css_selector(".select2-results")
        elem = driver.find_element_by_css_selector(".select2-selection")
        elem.click()
        results = driver.find_element_by_css_selector(".select2-results")
        assert results.is_displayed() is True
        elem = results.find_element_by_css_selector(".select2-results__option")
        elem.click()

        with pytest.raises(NoSuchElementException):
            error = driver.find_element_by_xpath("//body[@JSError]")
            pytest.fail(error.get_attribute("JSError"))

    def test_data_url(self):
        with pytest.raises(ValueError):
            HeavySelect2Widget()

        widget = HeavySelect2Widget(data_url="/foo/bar")
        assert widget.get_url() == "/foo/bar"

    def test_empty_option(self, db):
        # Empty options is only required for single selects
        # https://select2.github.io/options.html#allowClear
        single_select = self.form.fields["primary_genre"]
        assert single_select.required is False
        assert '<option value=""></option>' in single_select.widget.render(
            "primary_genre", None
        )

        multiple_select = self.multiple_form.fields["featured_artists"]
        assert multiple_select.required is False
        assert multiple_select.widget.allow_multiple_selected
        output = multiple_select.widget.render("featured_artists", None)
        assert '<option value=""></option>' not in output
        assert 'data-placeholder=""' in output

    def test_i18n(self):
        translation.activate("de")
        assert tuple(Select2Widget().media._js) == (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js",
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/i18n/de.js",
            "django_select2/django_select2.js",
        )

        translation.activate("en")
        assert tuple(Select2Widget().media._js) == (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js",
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/i18n/en.js",
            "django_select2/django_select2.js",
        )

        translation.activate("00")
        assert tuple(Select2Widget().media._js) == (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js",
            "django_select2/django_select2.js",
        )

        translation.activate("sr-cyrl")
        assert tuple(Select2Widget().media._js) == (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js",
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/i18n/sr-Cyrl.js",
            "django_select2/django_select2.js",
        )

        pytest.importorskip("django", minversion="2.0.4")

        translation.activate("zh-hans")
        assert tuple(Select2Widget().media._js) == (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js",
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/i18n/zh-CN.js",
            "django_select2/django_select2.js",
        )

        translation.activate("zh-hant")
        assert tuple(Select2Widget().media._js) == (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js",
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/i18n/zh-TW.js",
            "django_select2/django_select2.js",
        )

    def test_theme_setting(self, settings):
        settings.SELECT2_THEME = "classic"
        widget = self.widget_cls()
        assert 'data-theme="classic"' in widget.render("name", None)


class TestSelect2AdminMixin:
    def test_media(self):
        translation.activate("en")
        assert tuple(Select2AdminMixin().media._js) == (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js",
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/i18n/en.js",
            "django_select2/django_select2.js",
        )

        assert dict(Select2AdminMixin().media._css) == {
            "screen": [
                "admin/css/vendor/select2/select2.min.css",
                "admin/css/autocomplete.css",
            ]
        }


class TestSelect2MixinSettings:
    def test_default_media(self):
        sut = Select2Widget()
        result = sut.media.render()
        assert (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/js/select2.min.js"
            in result
        )
        assert (
            f"https://cdnjs.cloudflare.com/ajax/libs/select2/{settings.SELECT2_LIB_VERSION}/css/select2.min.css"
            in result
        )
        assert "django_select2/django_select2.js" in result

    def test_js_setting(self, settings):
        settings.SELECT2_JS = "alternate.js"
        sut = Select2Widget()
        result = sut.media.render()
        assert "alternate.js" in result
        assert "django_select2/django_select2.js" in result

    def test_empty_js_setting(self, settings):
        settings.SELECT2_JS = ""
        sut = Select2Widget()
        result = sut.media.render()
        assert "django_select2/django_select2.js" in result

    def test_css_setting(self, settings):
        settings.SELECT2_CSS = "alternate.css"
        sut = Select2Widget()
        result = sut.media.render()
        assert "alternate.css" in result

    def test_empty_css_setting(self, settings):
        settings.SELECT2_CSS = ""
        sut = Select2Widget()
        result = sut.media.render()
        assert "/select2.css" not in result

    def test_multiple_css_setting(self, settings):
        settings.SELECT2_CSS = ["select2.css", "select2-theme.css"]
        sut = Select2Widget()
        result = sut.media.render()
        assert "select2.css" in result
        assert "select2-theme.css" in result


class TestHeavySelect2Mixin(TestSelect2Mixin):
    url = reverse("heavy_select2_widget")
    form = forms.HeavySelect2WidgetForm(initial={"primary_genre": 1})
    widget_cls = HeavySelect2Widget

    def test_initial_data(self):
        assert "One" in self.form.as_p()

    def test_initial_form_class(self):
        widget = self.widget_cls(data_view="heavy_data_1", attrs={"class": "my-class"})
        assert "my-class" in widget.render("name", None)
        assert "django-select2" in widget.render("name", None)
        assert "django-select2-heavy" in widget.render("name", None), widget.render(
            "name", None
        )

    @pytest.mark.parametrize("code,name", SELECT2_TRANSLATIONS.items())
    def test_lang_attr(self, code, name):
        translation.activate(code)
        widget = self.widget_cls(data_view="heavy_data_1")
        assert f'lang="{name}"' in widget.render("name", None)

    def test_selected_option(self, db):
        not_required_field = self.form.fields["primary_genre"]
        assert not_required_field.required is False
        assert (
            '<option value="1" selected="selected">One</option>'
            in not_required_field.widget.render("primary_genre", 1)
            or '<option value="1" selected>One</option>'
            in not_required_field.widget.render("primary_genre", 1)
        ), (not_required_field.widget.render("primary_genre", 1))

    def test_many_selected_option(self, db, genres):
        field = HeavySelect2MultipleWidgetForm().fields["genres"]
        field.widget.choices = NUMBER_CHOICES
        widget_output = field.widget.render("genres", [1, 2])
        selected_option = (
            '<option value="{pk}" selected="selected">{value}</option>'.format(
                pk=1, value="One"
            )
        )
        selected_option_a = '<option value="{pk}" selected>{value}</option>'.format(
            pk=1, value="One"
        )
        selected_option2 = (
            '<option value="{pk}" selected="selected">{value}</option>'.format(
                pk=2, value="Two"
            )
        )
        selected_option2a = '<option value="{pk}" selected>{value}</option>'.format(
            pk=2, value="Two"
        )

        assert (
            selected_option in widget_output or selected_option_a in widget_output
        ), widget_output
        assert selected_option2 in widget_output or selected_option2a in widget_output

    def test_multiple_widgets(self, db, live_server, driver):
        driver.get(live_server + self.url)
        with pytest.raises(NoSuchElementException):
            driver.find_element_by_css_selector(".select2-results")

        elem1, elem2 = driver.find_elements_by_css_selector(".select2-selection")

        elem1.click()
        search1 = driver.find_element_by_css_selector(".select2-search__field")
        search1.send_keys("fo")
        result1 = (
            WebDriverWait(driver, 60)
            .until(
                expected_conditions.presence_of_element_located(
                    (By.CSS_SELECTOR, ".select2-results li:first-child")
                )
            )
            .text
        )

        elem2.click()
        search2 = driver.find_element_by_css_selector(".select2-search__field")
        search2.send_keys("fo")
        result2 = (
            WebDriverWait(driver, 60)
            .until(
                expected_conditions.presence_of_element_located(
                    (By.CSS_SELECTOR, ".select2-results li:first-child")
                )
            )
            .text
        )

        assert result1 != result2

        with pytest.raises(NoSuchElementException):
            error = driver.find_element_by_xpath("//body[@JSError]")
            pytest.fail(error.get_attribute("JSError"))

    def test_get_url(self):
        widget = self.widget_cls(data_view="heavy_data_1", attrs={"class": "my-class"})
        assert isinstance(widget.get_url(), str)

    def test_can_not_pickle(self):
        widget = self.widget_cls(data_view="heavy_data_1", attrs={"class": "my-class"})

        class NoPickle:
            pass

        widget.no_pickle = NoPickle()
        with pytest.raises(NotImplementedError):
            widget.set_to_cache()

    def test_theme_setting(self, settings):
        settings.SELECT2_THEME = "classic"
        widget = self.widget_cls(data_view="heavy_data_1")
        assert 'data-theme="classic"' in widget.render("name", None)


class TestModelSelect2Mixin(TestHeavySelect2Mixin):
    form = forms.AlbumModelSelect2WidgetForm(initial={"primary_genre": 1})
    multiple_form = forms.ArtistModelSelect2MultipleWidgetForm()

    def test_initial_data(self, genres):
        genre = genres[0]
        form = self.form.__class__(initial={"primary_genre": genre.pk})
        assert str(genre) in form.as_p()

    def test_label_from_instance_initial(self, genres):
        genre = genres[0]
        genre.title = genre.title.lower()
        genre.save()

        form = self.form.__class__(initial={"primary_genre": genre.pk})
        assert genre.title not in form.as_p(), form.as_p()
        assert genre.title.upper() in form.as_p()

    @pytest.fixture(autouse=True)
    def genres(self, genres):
        return genres

    def test_selected_option(self, db, genres):
        genre = genres[0]
        genre2 = genres[1]
        not_required_field = self.form.fields["primary_genre"]
        assert not_required_field.required is False
        widget_output = not_required_field.widget.render("primary_genre", genre.pk)
        selected_option = (
            '<option value="{pk}" selected="selected">{value}</option>'.format(
                pk=genre.pk, value=force_str(genre)
            )
        )
        selected_option_a = '<option value="{pk}" selected>{value}</option>'.format(
            pk=genre.pk, value=force_str(genre)
        )
        unselected_option = '<option value="{pk}">{value}</option>'.format(
            pk=genre2.pk, value=force_str(genre2)
        )

        assert (
            selected_option in widget_output or selected_option_a in widget_output
        ), widget_output
        assert unselected_option not in widget_output

    def test_selected_option_label_from_instance(self, db, genres):
        genre = genres[0]
        genre.title = genre.title.lower()
        genre.save()

        field = self.form.fields["primary_genre"]
        widget_output = field.widget.render("primary_genre", genre.pk)

        def get_selected_options(genre):
            return (
                '<option value="{pk}" selected="selected">{value}</option>'.format(
                    pk=genre.pk, value=force_str(genre)
                ),
                '<option value="{pk}" selected>{value}</option>'.format(
                    pk=genre.pk, value=force_str(genre)
                ),
            )

        assert all(o not in widget_output for o in get_selected_options(genre))
        genre.title = genre.title.upper()

        assert any(o in widget_output for o in get_selected_options(genre))

    def test_get_queryset(self):
        widget = ModelSelect2Widget()
        with pytest.raises(NotImplementedError):
            widget.get_queryset()
        widget.model = Genre
        assert isinstance(widget.get_queryset(), QuerySet)
        widget.model = None
        widget.queryset = Genre.objects.all()
        assert isinstance(widget.get_queryset(), QuerySet)

    def test_tag_attrs_Select2Widget(self):
        widget = Select2Widget()
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="0"' in output

    def test_custom_tag_attrs_Select2Widget(self):
        widget = Select2Widget(attrs={"data-minimum-input-length": "3"})
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="3"' in output

    def test_tag_attrs_ModelSelect2Widget(self):
        widget = ModelSelect2Widget(
            queryset=Genre.objects.all(), search_fields=["title__icontains"]
        )
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="2"' in output

    def test_tag_attrs_ModelSelect2TagWidget(self):
        widget = ModelSelect2TagWidget(
            queryset=Genre.objects.all(), search_fields=["title__icontains"]
        )
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="2"' in output

    def test_tag_attrs_HeavySelect2Widget(self):
        widget = HeavySelect2Widget(data_url="/foo/bar/")
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="2"' in output

    def test_custom_tag_attrs_ModelSelect2Widget(self):
        widget = ModelSelect2Widget(
            queryset=Genre.objects.all(),
            search_fields=["title__icontains"],
            attrs={"data-minimum-input-length": "3"},
        )
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="3"' in output

    def test_get_search_fields(self):
        widget = ModelSelect2Widget()
        with pytest.raises(NotImplementedError):
            widget.get_search_fields()

        widget.search_fields = ["title__icontains"]
        assert isinstance(widget.get_search_fields(), Iterable)
        assert all(isinstance(x, str) for x in widget.get_search_fields())

    def test_filter_queryset(self, genres):
        widget = TitleModelSelect2Widget(queryset=Genre.objects.all())
        assert widget.filter_queryset(None, genres[0].title[:3]).exists()

        widget = TitleModelSelect2Widget(
            search_fields=["title__icontains"], queryset=Genre.objects.all()
        )
        qs = widget.filter_queryset(
            None, " ".join([genres[0].title[:3], genres[0].title[3:]])
        )
        assert qs.exists()

    def test_filter_queryset__empty(self, genres):
        widget = TitleModelSelect2Widget(queryset=Genre.objects.all())
        assert widget.filter_queryset(None, genres[0].title[:3]).exists()

        widget = TitleModelSelect2Widget(
            search_fields=["title__icontains"], queryset=Genre.objects.all()
        )
        qs = widget.filter_queryset(None, "")
        assert qs.exists()

    def test_filter_queryset__startswith(self, genres):
        genre = Genre.objects.create(title="Space Genre")
        widget = TitleModelSelect2Widget(queryset=Genre.objects.all())
        assert widget.filter_queryset(None, genre.title).exists()

        widget = TitleModelSelect2Widget(
            search_fields=["title__istartswith"], queryset=Genre.objects.all()
        )
        qs = widget.filter_queryset(None, "Space Gen")
        assert qs.exists()

        qs = widget.filter_queryset(None, "Gen")
        assert not qs.exists()

    def test_filter_queryset__contains(self, genres):
        genre = Genre.objects.create(title="Space Genre")
        widget = TitleModelSelect2Widget(queryset=Genre.objects.all())
        assert widget.filter_queryset(None, genre.title).exists()

        widget = TitleModelSelect2Widget(
            search_fields=["title__contains"], queryset=Genre.objects.all()
        )
        qs = widget.filter_queryset(None, "Space Gen")
        assert qs.exists()

        qs = widget.filter_queryset(None, "NOT Gen")
        assert not qs.exists(), "contains works even if all bits match"

    def test_filter_queryset__multiple_fields(self, genres):
        genre = Genre.objects.create(title="Space Genre")
        widget = TitleModelSelect2Widget(queryset=Genre.objects.all())
        assert widget.filter_queryset(None, genre.title).exists()

        widget = TitleModelSelect2Widget(
            search_fields=[
                "title__startswith",
                "title__endswith",
            ],
            queryset=Genre.objects.all(),
        )
        qs = widget.filter_queryset(None, "Space")
        assert qs.exists()

        qs = widget.filter_queryset(None, "Genre")
        assert qs.exists()

    def test_model_kwarg(self):
        widget = ModelSelect2Widget(model=Genre, search_fields=["title__icontains"])
        genre = Genre.objects.last()
        result = widget.filter_queryset(None, genre.title)
        assert result.exists()

    def test_queryset_kwarg(self):
        widget = ModelSelect2Widget(
            queryset=Genre.objects.all(), search_fields=["title__icontains"]
        )
        genre = Genre.objects.last()
        result = widget.filter_queryset(None, genre.title)
        assert result.exists()

    def test_ajax_view_registration(self, client):
        widget = ModelSelect2Widget(
            queryset=Genre.objects.all(), search_fields=["title__icontains"]
        )
        widget.render("name", "value")
        url = reverse("django_select2:auto-json")
        genre = Genre.objects.last()
        response = client.get(
            url, data=dict(field_id=widget.field_id, term=genre.title)
        )
        assert response.status_code == 200, response.content
        data = json.loads(response.content.decode("utf-8"))
        assert data["results"]
        assert genre.pk in [result["id"] for result in data["results"]]

    def test_render(self):
        widget = ModelSelect2Widget(queryset=Genre.objects.all())
        widget.render("name", "value")
        cached_widget = cache.get(widget._get_cache_key())
        assert cached_widget["max_results"] == widget.max_results
        assert cached_widget["search_fields"] == tuple(widget.search_fields)
        qs = widget.get_queryset()
        assert isinstance(cached_widget["queryset"][0], qs.__class__)
        assert str(cached_widget["queryset"][1]) == str(qs.query)

    def test_get_url(self):
        widget = ModelSelect2Widget(
            queryset=Genre.objects.all(), search_fields=["title__icontains"]
        )
        assert isinstance(widget.get_url(), str)

    def test_custom_to_field_name(self):
        the_best_band_in_the_world = Artist.objects.create(title="Take That")
        groupie = Groupie.objects.create(obsession=the_best_band_in_the_world)
        form = forms.GroupieForm(instance=groupie)
        assert '<option value="Take That" selected>TAKE THAT</option>' in form.as_p()

    def test_empty_label(self, db):
        # Empty options is only required for single selects
        # https://select2.github.io/options.html#allowClear
        single_select = self.form.fields["primary_genre"]
        single_select.empty_label = "Hello World"
        assert single_select.required is False
        assert 'data-placeholder="Hello World"' in single_select.widget.render(
            "primary_genre", None
        )


class TestHeavySelect2TagWidget(TestHeavySelect2Mixin):
    def test_tag_attrs(self):
        widget = ModelSelect2TagWidget(
            queryset=Genre.objects.all(), search_fields=["title__icontains"]
        )
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="2"' in output
        assert 'data-tags="true"' in output
        assert "data-token-separators" in output

    def test_custom_tag_attrs(self):
        widget = ModelSelect2TagWidget(
            queryset=Genre.objects.all(),
            search_fields=["title__icontains"],
            attrs={"data-minimum-input-length": "3"},
        )
        output = widget.render("name", "value")
        assert 'data-minimum-input-length="3"' in output


class TestHeavySelect2MultipleWidget:
    url = reverse("heavy_select2_multiple_widget")
    form = forms.HeavySelect2MultipleWidgetForm()
    widget_cls = HeavySelect2MultipleWidget

    @pytest.mark.xfail(
        bool(os.environ.get("CI", False)),
        reason="https://bugs.chromium.org/p/chromedriver/issues/detail?id=1772",
    )
    def test_widgets_selected_after_validation_error(self, db, live_server, driver):
        driver.get(live_server + self.url)
        WebDriverWait(driver, 3).until(
            expected_conditions.presence_of_element_located((By.ID, "id_title"))
        )
        title = driver.find_element_by_id("id_title")
        title.send_keys("fo")
        genres, fartists = driver.find_elements_by_css_selector(
            ".select2-selection--multiple"
        )
        genres.click()
        genres.send_keys("o")  # results are Zero One Two Four
        # select second element - One
        driver.find_element_by_css_selector(".select2-results li:nth-child(2)").click()
        genres.submit()
        # there is a ValidationError raised, check for it
        errstring = (
            WebDriverWait(driver, 3)
            .until(
                expected_conditions.presence_of_element_located(
                    (By.CSS_SELECTOR, "ul.errorlist li")
                )
            )
            .text
        )
        assert errstring == "Title must have more than 3 characters."
        # genres should still have One as selected option
        result_title = driver.find_element_by_css_selector(
            ".select2-selection--multiple li"
        ).get_attribute("title")
        assert result_title == "One"


class TestAddressChainedSelect2Widget:
    url = reverse("model_chained_select2_widget")
    form = forms.AddressChainedSelect2WidgetForm()

    def test_widgets_selected_after_validation_error(
        self, db, live_server, driver, countries, cities
    ):
        driver.get(live_server + self.url)

        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-selection--single")
            )
        )
        (
            country_container,
            city_container,
            city2_container,
        ) = driver.find_elements_by_css_selector(".select2-selection--single")

        # clicking city select2 lists all available cities
        city_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li")
            )
        )
        city_options = driver.find_elements_by_css_selector(".select2-results li")
        city_names_from_browser = {option.text for option in city_options}
        city_names_from_db = set(City.objects.values_list("name", flat=True))
        assert len(city_names_from_browser) == City.objects.count()
        assert city_names_from_browser == city_names_from_db

        # selecting a country really does it
        country_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li:nth-child(2)")
            )
        )
        country_option = driver.find_element_by_css_selector(
            ".select2-results li:nth-child(2)"
        )
        country_name = country_option.text
        country_option.click()
        assert country_name == country_container.text

        # clicking city select2 lists reduced list of cities belonging to the country
        city_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li")
            )
        )
        city_options = driver.find_elements_by_css_selector(".select2-results li")
        city_names_from_browser = {option.text for option in city_options}
        city_names_from_db = set(
            Country.objects.get(name=country_name).cities.values_list("name", flat=True)
        )
        assert len(city_names_from_browser) != City.objects.count()
        assert city_names_from_browser == city_names_from_db

        # selecting a city reaaly does it
        city_option = driver.find_element_by_css_selector(
            ".select2-results li:nth-child(2)"
        )
        city_name = city_option.text
        city_option.click()
        assert city_name == city_container.text

        # clicking country select2 lists reduced list to the only country available to the city
        country_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li")
            )
        )
        country_options = driver.find_elements_by_css_selector(".select2-results li")
        country_names_from_browser = {option.text for option in country_options}
        country_names_from_db = {City.objects.get(name=city_name).country.name}
        assert len(country_names_from_browser) != Country.objects.count()
        assert country_names_from_browser == country_names_from_db

    def test_dependent_fields_clear_after_change_parent(
        self, db, live_server, driver, countries, cities
    ):
        driver.get(live_server + self.url)
        (
            country_container,
            city_container,
            city2_container,
        ) = driver.find_elements_by_css_selector(".select2-selection--single")

        # selecting a country really does it
        country_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li:nth-child(2)")
            )
        )
        country_option = driver.find_element_by_css_selector(
            ".select2-results li:nth-child(2)"
        )
        country_name = country_option.text
        country_option.click()
        assert country_name == country_container.text

        # selecting a city2
        city2_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li")
            )
        )
        city2_option = driver.find_element_by_css_selector(
            ".select2-results li:nth-child(2)"
        )
        city2_name = city2_option.text
        city2_option.click()
        assert city2_name == city2_container.text

        # change a country
        country_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li:nth-child(3)")
            )
        )
        country_option = driver.find_element_by_css_selector(
            ".select2-results li:nth-child(3)"
        )
        country_name = country_option.text
        country_option.click()
        assert country_name == country_container.text

        # check the value in city2
        city2_container.click()
        WebDriverWait(driver, 60).until(
            expected_conditions.presence_of_element_located(
                (By.CSS_SELECTOR, ".select2-results li")
            )
        )
        assert city2_container.text == ""



#!/usr/bin/env python3
"""Set the version in NPM's package.json to match the git tag."""
import json
import os

if __name__ == "__main__":
    with open("package.json", "r+") as f:
        data = json.load(f)
        f.seek(0)
        data["version"] = os.environ["GITHUB_REF"].rsplit("/")[-1]
        json.dump(data, f)
        f.truncate()



"""
Django-Select2 URL configuration.

Add `django_select` to your ``urlconf`` **if** you use any 'Model' fields::

    from django.urls import path


    path('select2/', include('django_select2.urls')),

"""
from django.urls import path

from .views import AutoResponseView

app_name = "django_select2"

urlpatterns = [
    path("fields/auto.json", AutoResponseView.as_view(), name="auto-json"),
]
